<?php if (!defined('PmWiki')) exit();
/* dublincore.php -- provides Simple Dublin Core metadata for PmWiki pages
 * Copyright 2004, Evan Prodromou <evan@pigdog.org>.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

$DublinCoreVersion = "0.1";

/* BEGIN ADMIN-VARIABLES. */

/* Which MIME type to use for RDF.*/

SDV($RdfMimeType, 'text/xml');

/* An 'About' page. */

SDV($AboutPage, "$DefaultGroup.$WikiTitle");

/* A page that gives the copyright policy for the site. */

SDV($CopyrightPage, "$DefaultGroup.CopyrightPolicy");

/* An URL for the license of your site. */

SDV($LicenseUrl, NULL);

/* Some plain text to explain your copyright policy. */

SDV($CopyrightText, NULL);

/* END ADMIN-VARIABLES. */

$HandleActions['dc'] = 'HandleDc';

/* We show a <link> element for browsing. */

if ($action == 'browse') {
    if ($pagename == '') $pagename = $DefaultPage;
    $pageurl = FmtPageName('$PageUrl', $pagename);
    $RdfUrl = $pageurl . "?action=dc";
    $HTMLHeaderFmt[] = "<link rel='meta' type='{$RdfMimeType}' href='{$RdfUrl}' />\n";
}

/* Handles outputting the Dublin Core RDF. */

function HandleDc($pagename) {
    
    global $WikiTitle, $RdfMimeType;
    global $AboutPage;
    global $CopyrightPage, $LicenseUrl, $CopyrightText;
    
    $page = RetrieveAuthPage($pagename,"read");
    if (!$page) Abort("Invalid page name");
    Lock(0);
    header("Content-Type: {$RdfMimeType}");
    
    if ($pagename == '') $pagename = $DefaultPage;
    $pageurl = FmtPageName('$PageUrl', $pagename);
    
    $last_author = @$page['author'];
    
    DcPrologue($pageurl);
    DcPageOrString('publisher', $AboutPage, $WikiTitle);
    DcElement('title', $pagename);
    DcElement('date', strftime('%Y-%m-%d', $page['time']));
    DcPerson('creator', $last_author);
    DcElement('type', 'Text');
    DcElement('format', 'text/html');
    DcElement('identifier', $pageurl);

    $contributors = array();

    foreach ($page as $k=>$v) {
	if (preg_match('/^author\:\d+/', $k)) {
	    if (strcmp($v, $last_author) != 0 &&
		!in_array($v, $contributors)) {
		$contributors[] = $v;
	    }
	}
    }

    foreach ($contributors as $contrib) {
	DcPerson('contributor', $contrib);
    }

    if (isset($CopyrightPage) && PageExists($CopyrightPage)) {
	DcPage('rights', $CopyrightPage);
    } else if (isset($LicenseUrl)) {
	DcUrl('rights', $LicenseUrl);
    } else if (isset($CopyrightText)) {
	DcElement('rights', $CopyrightText);
    }

    DcEpilogue();
}

/* The beginning boilerplate. */

function DcPrologue($pageurl) {
    echo "<?xml version='1.0' encoding='iso-8859-1' ?>
<!DOCTYPE rdf:RDF PUBLIC \"-//DUBLIN CORE//DCMES DTD 2002/07/31//EN\" \"http://dublincore.org/documents/2002/07/31/dcmes-xml/dcmes-xml-dtd.dtd\">
	   
	   <rdf:RDF xmlns=\"http://web.resource.org/cc/\"
	   xmlns:dc=\"http://purl.org/dc/elements/1.1/\">
	   
	   <rdf:Description rdf:about=\"$pageurl\">
	   ";
}  

/* Output one Dublin Core element. */

function DcElement($name, $value) {
    echo "<dc:{$name}>{$value}</dc:{$name}>\n";
}

function DcPage($name, $page) {
    DcUrl($name, FmtPageName('$PageUrl', $page));
}

function DcUrl($name, $url) {
    echo "<dc:{$name} rdf:resource=\"{$url}\" />\n";
}

function DcPageOrString($name, $page, $string) {
    if (isset($page) && PageExists($page)) {
       DcPage($name, $page);
    } else {
	DcElement($name, $string);
    }
}

/* For a person, we either want a link to their user page,
 * if it exists, or their user name spaced out, or just "Anonymous"
 * if no person was provided. */

function DcPerson($name, $person) {
    if (!$person) {
	DcElement($name, 'Anonymous');
    } else {
	global $AuthorGroup;
	$profile = "$AuthorGroup.$person";
	DcPageOrString($name, $profile, AsSpaced($person));
    }
}

/* The ending boilerplate. */

function DcEpilogue() {
    echo "
	   </rdf:Description>
	   </rdf:RDF>
	   ";
}

?>
